// Copyright Sebastian Wiesner <sebastian@swsnr.de>
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0.If a copy of the MPL was not distributed with this
// file, You can obtain one at https://mozilla.org/MPL/2.0/.
//
// Alternatively, the contents of this file may be used under the terms
// of the GNU General Public License Version 2 or later, as described below:
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
import GObject from "gi://GObject";
import GLib from "gi://GLib";
import Gio from "gi://Gio";
import Gtk from "gi://Gtk";
import Adw from "gi://Adw";
import { ExtensionPreferences } from "resource:///org/gnome/Shell/Extensions/js/extensions/prefs.js";
const getTemplate = (name) => GLib.uri_resolve_relative(import.meta.url, `ui/${name}.ui`, GLib.UriFlags.NONE);

const LICENSE = `Copyright Sebastian Wiesner <sebastian@swsnr.de>

This programm is subject to the terms of the Mozilla Public
License, v. 2.0.If a copy of the MPL was not distributed with this
file, You can obtain one at https://mozilla.org/MPL/2.0/.

Alternatively, this program may be used under the terms
of the GNU General Public License Version 2 or later, as described below:

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.`;

const ClockPage = GObject.registerClass({
    GTypeName: "UTCClockClockPage",
    Template: getTemplate("ClockPage"),
    InternalChildren: ["dateTimeFormat", "previewLabel"],
}, class UTCClockClockPage extends Adw.PreferencesPage {
    constructor(settings) {
        super();
        const children = this;
        settings.bind("clock-format", children._dateTimeFormat, "text", Gio.SettingsBindFlags.DEFAULT);
        const previewDateTime = GLib.DateTime.new(GLib.TimeZone.new_utc(), 2006, 1, 2, 15, 4, 5);

        const updatePreview = () => {
            const currentFormat = settings.get_string("clock-format");
            if (currentFormat) {
                const preview = previewDateTime.format(currentFormat) ?? "";
                children._previewLabel.set_markup(`<span style="italic">${preview}</span>`);
            }
            else {
                children._previewLabel.set_text("");
            }
        };
        updatePreview();
        settings.connect("changed::clock-format", updatePreview);
    }
});

const AboutPage = GObject.registerClass({
    GTypeName: "UTCClockAboutPage",
    Template: getTemplate("AboutPage"),
    InternalChildren: [
        "extensionName",
        "extensionVersion",
        "linkGithub",
        "linkIssues",
        "extensionLicense",
    ],
}, class UTCClockAboutPage extends Adw.PreferencesPage {
    constructor(metadata) {
        super();
        const children = this;
        children._extensionName.set_text(metadata.name);
        if (metadata["version-name"]) {
            children._extensionVersion.set_text(metadata["version-name"]);
        }
        else {
            children._extensionVersion.visible = false;
        }
        if (metadata.url) {
            children._linkGithub.set_uri(metadata.url);
            children._linkIssues.set_uri(`${metadata.url}/issues`);
        }
        else {
            children._linkGithub.visible = false;
            children._linkIssues.visible = false;
        }
        children._extensionLicense.buffer.set_text(LICENSE, -1);
    }
});

export default class UTCClockPreferences extends ExtensionPreferences {
    fillPreferencesWindow(window) {
        // Add our icons directory to the Gtk theme path, so that we're able to use
        // our icons in Adwaita widgets.
        const iconTheme = Gtk.IconTheme.get_for_display(window.get_display());
        const iconsDirectory = this.metadata.dir.get_child("icons").get_path();
        if (iconsDirectory === null) {
            throw new Error("Failed to get path for icon directory");
        }
        iconTheme.add_search_path(iconsDirectory);
        const settings = this.getSettings();
        // Attach the settings object to the window to keep it alive while the window is alive.
        window._settings = settings;
        window.add(new ClockPage(settings));
        window.add(new AboutPage(this.metadata));
    }
}
